﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DMTTest
{
	public partial class MainForm : Form
	{
		private enum State
		{
			Params,
			Test,
			Results,

			Last
		}
		private State state = State.Params;
		private Panel currentPanel = null;
		private System.Collections.Generic.List<TestItem> answers = 
			new System.Collections.Generic.List<TestItem>();
		private System.DateTime timeStart;

		public MainForm()
		{
			this.InitializeComponent();
		}

		private void ShowPanel()
		{
			Logger.WriteEvent(Properties.Resources.ResourceManager.GetString("state" + this.state.ToString()));
			Panel panel = null;
			switch (this.state)
			{
				case State.Params:
					panel = this.pnlParams;
					break;
				case State.Test:
					panel = this.pnlTest;
					this.answers.Clear();
					this.timeStart = System.DateTime.Now;
					break;
				case State.Results:
					this.pnlResults.SetResults(this.timeStart, System.DateTime.Now, this.answers);
					panel = this.pnlResults;
					break;
			}
			if (this.currentPanel != null)
			{
				this.currentPanel.Deactivate();
			}
			panel.Activate();
			panel.Visible = true;
			if (this.currentPanel != null)
			{
				this.currentPanel.Visible = false;
			}
			panel.Focus();
			this.currentPanel = panel;
			this.btnPause.Enabled = this.btnPause.Visible = this.state == State.Test;
			this.SetPauseText();
			this.btnNext.Text = Properties.Resources.ResourceManager.GetString(this.btnNext.Name + this.state + "Text");
		}

		private void Next()
		{
			++this.state;
			if (this.state >= State.Last)
			{
				this.state = State.Params;
			}
		}
		private void SetPauseText()
		{
			this.btnPause.Text =
				Properties.Resources.ResourceManager.GetString(
					this.btnPause.Name
					+ (this.pnlTest.IsPaused ? "Continue" : "Pause")
					+ "Text"
				);
		}
		private void PauseToggle()
		{
			this.pnlTest.PauseToggle();
			this.SetPauseText();
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			try
			{
				this.ShowPanel();
			}
			catch (Exception ex)
			{
				Logger.WriteError(ex);
			}
		}

		private void btnNext_Click(object sender, EventArgs e)
		{
			try
			{
				this.Next();
				this.ShowPanel();
			}
			catch (Exception ex)
			{
				Logger.WriteError(ex);
			}
		}

		private void btnPause_Click(object sender, EventArgs e)
		{
			try
			{
				this.PauseToggle();
			}
			catch (Exception ex)
			{
				Logger.WriteError(ex);
			}
		}

		private void pnlTest_StateEvent(object sender, Test.StateEventArgs e)
		{
			try
			{
				switch (e.State)
				{
					case Test.State.Answer:
						this.answers.Add(e.Question);
						Logger.WriteResult(e.Question.ToString());
						break;
					case Test.State.Paused:
						//this.btnPause.Focus();
						this.PauseToggle();
						break;
					case Test.State.Complete:
					case Test.State.Stopped:
						this.Next();
						this.ShowPanel();
						break;
				}
			}
			catch (Exception ex)
			{
				Logger.WriteError(ex);
			}
		}
	}
}
