﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace DMTTest
{
	public partial class Results : Panel
	{
		private System.DateTime start;
		private System.DateTime stop;
		private System.Collections.Generic.List<TestItem> answers;

		public Results()
		{
			InitializeComponent();
		}

		internal void SetResults(System.DateTime start, System.DateTime stop, System.Collections.Generic.List<TestItem> answers)
		{
			this.start = start;
			this.stop = stop;
			this.answers = answers;
		}

		internal override void Activate()
		{
			const int resLength = 40;
			string s = this.start.ToString(Properties.Resources.CommonTimeFormat);
			this.lblStartValue.Text = s;
			this.LogResult(this.lblStart, resLength, s);
			s = this.stop.ToString(Properties.Resources.CommonTimeFormat);
			this.lblStopValue.Text = s;
			this.LogResult(this.lblStop, resLength, s);
			System.TimeSpan t = new System.TimeSpan(0);
			System.TimeSpan tgood = new System.TimeSpan(0);
			int good = 0;
			int timedout = 0;
			this.lvResults.Items.Clear();
			foreach (TestItem ti in this.answers)
			{
				System.Windows.Forms.ListViewItem lvi =
					new ListViewItem(
						new string[] {
							ti.CompleteText,
							ti.QuestionText,
							ti.AnswerText,
							ti.IsTrueText,
							ti.ResultText,
							ti.DurationText
						}, 
						-1, 
						ti.IsTrue
						? System.Drawing.Color.Green
						: ti.Answer.HasValue 
						  ? System.Drawing.Color.Red 
						  : System.Drawing.Color.Brown,
						this.lvResults.BackColor,
						this.lvResults.Font
					);
				this.lvResults.Items.Add(lvi);
				if (ti.Answer.HasValue)
				{
					t += ti.Duration;
					if (ti.IsTrue)
					{
						tgood += ti.Duration;
						++good;
					}
					continue;
				}
				++timedout;
			}
			if (this.answers.Count > 0)
			{
				s = this.answers.Count.ToString();
				this.lblAnswersValue.Text = s;
				this.LogResult(this.lblAnswers, resLength, s);
				s = good.ToString();
				this.lblAnswersTrueValue.Text = s;
				string s1 = (((float)good) / this.answers.Count).ToString("(0.00%)");
				this.lblAnswersTruePercent.Text = s1;
				this.LogResult(this.lblAnswersTrue, resLength, s + " " + s1);
				s = (this.answers.Count - good - timedout).ToString();
				this.lblAnswersFalseValue.Text = s;
				s1 = (((float)(this.answers.Count - good - timedout)) / this.answers.Count).ToString("(0.00%)");
				this.lblAnswersFalsePercent.Text = s1;
				this.LogResult(this.lblAnswersFalse, resLength, s + " " + s1);
				s = timedout.ToString();
				this.lblAnswersTimedOutValue.Text = s;
				s1 = (((float)timedout) / this.answers.Count).ToString("(0.00%)");
				this.lblAnswersTimedOutPercent.Text = s1;
				this.LogResult(this.lblAnswersTimedOut, resLength, s + " " + s1);
				s = (Config.TimeSpan2Decimal(t) / this.answers.Count).ToString("0.000");
				this.lblAverageTimeValue.Text = s;
				this.lblAverageTimeUnits.Visible = true;
				this.LogResult(this.lblAverageTime, resLength, s);
				if (good == 0)
				{
					this.lblAverageTimeTrueValue.Text = string.Empty;
					this.lblAverageTimeTrueUnits.Visible = false;
					return;
				}
				s = (Config.TimeSpan2Decimal(tgood) / good).ToString("0.000");
				this.lblAverageTimeTrueValue.Text = s;
				this.lblAverageTimeTrueUnits.Visible = true;
				this.LogResult(this.lblAverageTimeTrue, resLength, s);
				return;
			}
			this.lblAverageTimeUnits.Visible 
				= this.lblAverageTimeTrueUnits.Visible 
				= false;
			this.lblAnswersValue.Text
				= this.lblAnswersTrueValue.Text = this.lblAnswersTruePercent.Text
				= this.lblAnswersFalseValue.Text = this.lblAnswersFalsePercent.Text
				= this.lblAnswersTimedOutValue.Text = this.lblAnswersTimedOutPercent.Text
				= this.lblAverageTimeValue.Text
				= this.lblAverageTimeTrueValue.Text
				= string.Empty;
			Logger.WriteResult(Properties.Resources.NoQuestionsAnswered);
		}

		private void LogResult(System.Windows.Forms.Control c, int length, string text)
		{
			length -= c.Text.Length;
			if (length <= 0)
			{
				length = 1;
			}
			Logger.WriteResult(c.Text + new string(' ', length) + text);
		}

		private void btn_Click(object sender, EventArgs e)
		{
			try
			{
				System.Windows.Forms.Button b = sender as System.Windows.Forms.Button;
				if (null == b)
				{
					return;
				}
				if (b == this.btnCopy)
				{
					int start = this.tbxLog.SelectionStart;
					int length = this.tbxLog.SelectionLength;
					this.tbxLog.SuspendLayout();
					this.tbxLog.SelectAll();
					this.tbxLog.Copy();
					this.tbxLog.Select(start, length);
					this.tbxLog.ResumeLayout(false);
					return;
				}
				if (b == this.btnClear)
				{
					this.tbxLog.Text = string.Empty;
					return;
				}
			}
			catch (System.Exception ex)
			{
				Logger.WriteError(ex);
			}
		}
	}
}
