﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace DMTTest
{
	public partial class Test : Panel
	{
		internal enum State
		{
			Stopped,
			Question,
			Answer,
			Paused,
			Complete
		}

		public class StateEventArgs
		{
			internal Test.State State;
			internal TestItem Question;
			internal StateEventArgs(Test.State state, TestItem question)
			{
				this.State = state;
				this.Question = question;
			}
		}
		public delegate void StateEventHandler(object sender, StateEventArgs e);

		private State state = State.Stopped;
		private System.TimeSpan questionTime;
		private System.DateTime questionStart;
		private TestItem question;
		private int trueAswers;
		private System.Random rndTest = new System.Random(System.DateTime.Now.Millisecond);

		public event StateEventHandler StateEvent;

		public Test()
		{
			this.InitializeComponent();
			this.InitializeButtons();
		}

		private void InitializeButtons()
		{
			for (int c = 0; c < 10; ++c)
			{
				for (int r = 0; r < 10; ++r)
				{
					System.Windows.Forms.Button b = new System.Windows.Forms.Button();
					b.AutoSize = true;
					b.AutoSizeMode = System.Windows.Forms.AutoSizeMode.GrowAndShrink;
					b.Enabled = false;
					b.Dock = System.Windows.Forms.DockStyle.Fill;
					b.Margin = new System.Windows.Forms.Padding(1);
					b.Padding = new System.Windows.Forms.Padding(0);
					b.TextAlign = ContentAlignment.MiddleCenter;
					b.Tag = c + r * 10;
					b.Text = b.Tag.ToString();
					b.Name = "btn" + b.Text;
					b.Click += new System.EventHandler(btnN_Click);
					this.pnlTest.Controls.Add(b, c, r + 1);
				}
			}
		}

		internal bool IsPaused
		{
			get
			{
				return this.state == State.Paused;
			}
		}

		internal override void Activate()
		{
			this.Start();
			#region dirty hack for mono
			this.Pause();
			this.Question();
			#endregion //dirty hack for mono
		}

		internal override void Deactivate()
		{
			this.Stop();
		}

		internal void Start()
		{
			this.questionTime = Config.Time;
			this.LogQuestionTime();
			this.trueAswers = Config.Answers;
			this.Question();
		}
		internal void Stop()
		{
			this.state = State.Stopped;
			this.Timer();
			this.question = null;
			this.Output();
			this.Input();
		}
		internal void Answer()
		{
			int a;
			if (int.TryParse(this.tbxAnswer.Text, out a))
			{
				this.Answer(a);
			}
		}
		internal void PauseToggle()
		{
			if (this.state == State.Paused)
			{
				Logger.WriteEvent("Resume");
				this.Question();
				return;
			}
			Logger.WriteEvent("Pause");
			this.Pause();
		}

		private void Question()
		{
			this.state = State.Question;
			TestItem.OpType op = (TestItem.OpType)this.rndTest.Next(2);
			int left = this.rndTest.Next(9) + 1;
			int right = Config.Multipliers[this.rndTest.Next(Config.Multipliers.Length)];
			this.question = new TestItem(op, left, right);
			this.Output();
			this.Input();
			this.Timer();
		}
		private void Answer(int? answer)
		{
			this.state = State.Answer;
			if (this.question != null)
			{
				this.question.SetAnswer(this.questionStart, System.DateTime.Now, answer);
				if (this.question.IsTrue)
				{
					--this.trueAswers;
				}
			}
			this.Output();
			this.Input();
			this.Timer();
			if (this.StateEvent != null)
			{
				this.StateEvent(this, new StateEventArgs(State.Answer, this.question));
			}
		}
		private void Pause()
		{
			this.state = State.Paused;
			this.Timer();
			this.question = null;
			this.Output();
			this.Input();
		}

		private void Output()
		{
			this.pnlTest.SuspendLayout();
			this.lblQuestion.Visible = this.state != State.Stopped;
			this.lblAnswer.Visible =
				this.tbxAnswer.Visible =
				this.lblIsTrue.Visible =
				this.lblResult.Visible =
				this.state != State.Stopped && this.state != State.Paused;
			this.tbxAnswer.Enabled = this.state != State.Stopped || this.state != State.Paused;
			this.lblQuestion.Text = 
				this.tbxAnswer.Text = 
				this.lblIsTrue.Text = 
				this.lblResult.Text = string.Empty;
			this.tbxAnswer.BackColor = System.Drawing.SystemColors.Window;
			this.tbxAnswer.ForeColor = 
				this.lblIsTrue.ForeColor =
				this.lblResult.ForeColor =
				System.Drawing.SystemColors.WindowText;
			switch (this.state)
			{
				case State.Question:
					if (this.question != null)
					{
						this.lblQuestion.Text = this.question.QuestionText;
					}
					break;
				case State.Answer:
					if (this.question != null)
					{
						this.lblQuestion.Text = this.question.QuestionText;
						this.tbxAnswer.Text = this.question.AnswerText;
						this.tbxAnswer.Select(this.tbxAnswer.Text.Length, 0);
						this.lblResult.Text = this.question.ResultText;
						this.tbxAnswer.ForeColor = System.Drawing.Color.White;
						this.lblIsTrue.Text = this.question.IsTrueText;
						this.tbxAnswer.BackColor = 
							this.lblIsTrue.ForeColor = 
							this.lblResult.ForeColor = 
							this.question.IsTrue
							? System.Drawing.Color.Green 
							: this.question.Answer.HasValue 
							  ? System.Drawing.Color.Red 
							  : System.Drawing.Color.Brown;
					}
					break;
				case State.Paused:
					this.lblQuestion.Text = Properties.Resources.lblQuestionPausedText;
					break;
			}
			this.pnlTest.ResumeLayout();
		}
		private void Input()
		{
			bool enable = this.state == State.Question;
			this.pnlTest.SuspendLayout();
			foreach (System.Windows.Forms.Control c in this.pnlTest.Controls)
			{
				if (c is System.Windows.Forms.Button)
				{
					c.Enabled = enable;
				}
			}
			this.tbxAnswer.ReadOnly = !enable;
			this.tbxAnswer.Focus();
			this.pnlTest.ResumeLayout();
		}
		private void Timer()
		{
			this.tmrTest.Enabled = false;
			if (this.state == State.Stopped || this.state == State.Paused)
			{
				return;
			}
			this.tmrTest.Interval = 
				this.question != null && this.question.IsTrue 
				? (int)Config.TimeResTrue.TotalMilliseconds 
				: (int)Config.TimeResFalse.TotalMilliseconds;
			if (this.state == State.Question)
			{
				if (this.trueAswers == 0)
				{
					this.trueAswers = Config.Answers;
					this.questionTime -= Config.TimeDec;
					if (this.questionTime <= Config.TimeMin)
					{
						if (this.StateEvent != null)
						{
							this.StateEvent(this, new StateEventArgs(State.Complete, this.question));
						}
						return;
					}
					this.LogQuestionTime();
				}
				this.tmrTest.Interval = (int)this.questionTime.TotalMilliseconds;
				this.questionStart = System.DateTime.Now;
			}
			this.tmrTest.Enabled = true;
		}

		private void LogQuestionTime()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Params));
			Logger.WriteInfo(
				string.Format(
					resources.GetString("lblTime.Text")
					+ " {0:0.000} "
					+ resources.GetString("lblTimeUnits.Text"),
					Config.TimeSpan2Decimal(this.questionTime)
				)
			);
		}

		private void btnN_Click(object sender, EventArgs e)
		{
			try
			{
				this.Answer((int)((System.Windows.Forms.Button)sender).Tag);
			}
			catch (Exception ex)
			{
				Logger.WriteError(ex);
			}
		}

		private void tmrTest_Tick(object sender, EventArgs e)
		{
			try
			{
				this.tmrTest.Enabled = false;
				if (this.state == State.Question)
				{
					this.Answer(null);
					return;
				}
				this.Question();
			}
			catch (Exception ex)
			{
				Logger.WriteError(ex);
			}
		}

		private void tbxAnswer_KeyPress(object sender, KeyPressEventArgs e)
		{
			try
			{
				if (e.KeyChar == ' ' && this.StateEvent != null)
				{
					e.Handled = true;
					this.StateEvent(this, new StateEventArgs(State.Paused, this.question));
					return;
				}
				if (e.KeyChar == 27 && this.StateEvent != null)
				{
					e.Handled = true;
					this.StateEvent(this, new StateEventArgs(State.Stopped, this.question));
					return;
				}
				if (this.tbxAnswer.ReadOnly)
				{
					return;
				}
				if (e.KeyChar == '\r')
				{
					e.Handled = true;
					this.Answer();
					return;
				}
				e.Handled = !(
					char.IsDigit(e.KeyChar) 
					|| char.IsControl(e.KeyChar)
				);
			}
			catch (Exception ex)
			{
				Logger.WriteError(ex);
			}
		}
	}
}
