﻿using System;
using System.Collections.Generic;
using System.Text;

namespace DMTTest
{
	public class TestItem
	{

		internal enum OpType
		{
			Mul,
			Div
		}
		internal OpType Operation;
		internal int Left;
		internal int Right;
		internal int? Answer = null;
		internal System.TimeSpan Duration;
		internal System.DateTime Complete;

		internal TestItem(OpType operation, int left, int right)
		{
			this.Operation = operation;
			this.Left = left;
			this.Right = right;
		}

		internal void SetAnswer(System.DateTime start, System.DateTime stop, int? answer)
		{
			this.Answer = answer;
			this.Duration = stop - start;
			this.Complete = stop;
		}

		internal int Result
		{
			get
			{
				return
					this.Operation == OpType.Mul
						? this.Left * this.Right
						: this.Left;
			}
		}
		internal bool IsTrue
		{
			get
			{
				if (this.Answer.HasValue)
				{
					return this.Result == this.Answer;
				}
				return false;
			}
		}

		internal string IsTrueText
		{
			get
			{
				return
					this.IsTrue
					? Properties.Resources.lblIsTrueTrueText
					: this.Answer.HasValue
					  ? Properties.Resources.lblIsTrueFalseText
					  : Properties.Resources.lblIsTrueTimedOutText;
			}
		}

		internal string QuestionText
		{
			get
			{
				return 
					this.Operation == OpType.Mul 
					? string.Format("{0,2} {2} {1,2}", this.Left, this.Right, Properties.Resources.OpSignMul)
					: string.Format("{0,2} {2} {1,2}", this.Left * this.Right, this.Right, Properties.Resources.OpSignDiv);
			}
		}
		internal string ResultText
		{
			get
			{
				return string.Format("{0,2}", this.Result);
			}
		}
		internal string AnswerText
		{
			get
			{
				return
					this.Answer.HasValue
					? string.Format("{0,2}", this.Answer.Value)
					: Properties.Resources.AnswerEmptyText;
			}
		}
		internal string DurationText
		{
			get
			{
				return string.Format("{0,6:0.000}", this.Duration.Seconds + this.Duration.Milliseconds / 1000f);
			}
		}
		internal string CompleteText
		{
			get
			{
				return this.Complete.ToString(Properties.Resources.CommonTimeFormat);
			}
		}

		public override string ToString()
		{
			System.Text.StringBuilder sb = new System.Text.StringBuilder(this.QuestionText);
			sb.Append(" = ");
			sb.Append(this.AnswerText);
			sb.Append(" ");
			sb.Append(this.IsTrue ? "+" : "-");
			sb.Append(" (");
			sb.Append(this.ResultText);
			sb.Append(") ");
			sb.Append(this.DurationText);
			return sb.ToString();
		}
	}
}
