﻿using System;
using System.Collections.Generic;
using System.Text;

namespace DMTTest
{
	public sealed class WindowLogger : System.Diagnostics.TraceListener
	{
		private System.Windows.Forms.TextBox log;

		public WindowLogger(string logName)
			: base(logName)
		{
			this.SetLogByName(logName);
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				this.log = null;
			}
			base.Dispose(disposing);
		}

		public override void Write(string message)
		{
			if (null == this.log)
			{
				return;
			}
			this.log.Text += message;
			//this.log.ScrollToCaret();
			System.Windows.Forms.Application.DoEvents();
		}

		public override void WriteLine(string message)
		{
			this.Write(message + System.Environment.NewLine);
		}

		private void SetLogByName(string logName)
		{
			foreach (System.Windows.Forms.Form f in System.Windows.Forms.Application.OpenForms)
			{
				System.Windows.Forms.TextBox log = FindLogTexBox(f, logName);
				if (null != log)
				{
					this.log = log;
					this.log.Text = string.Empty;
					return;
				}
			}
		}

		private static System.Windows.Forms.TextBox FindLogTexBox(System.Windows.Forms.Control c, string name)
		{
			if (c is System.Windows.Forms.TextBox && c.Name == name)
			{
				return (System.Windows.Forms.TextBox)c;
			}
			foreach (System.Windows.Forms.Control cc in c.Controls)
			{
				System.Windows.Forms.TextBox log = FindLogTexBox(cc, name);
				if (null != log)
				{
					return log;
				}
			}
			return null;
		}
	}
}
